#==============================================================================
#
#  $Id: attchmt.py,v 1.3 2000/05/26 00:17:38 mike Exp $
#
"""
   Attachment viewer window and its widgets.
"""
#
#  Copyright (C) 1999 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#  $Log: attchmt.py,v $
#  Revision 1.3  2000/05/26 00:17:38  mike
#  Support for "7bit" transfer encodings.
#
#  Revision 1.2  2000/05/25 01:26:02  mike
#  Fixed writing of non-encoded attachments; added standard "enter key" and
#  "left key" behavior.
#
#  Revision 1.1  2000/05/11 01:09:54  mike
#  Added attachment processing.
#
#
#==============================================================================


import string
from spug.curses.ModalWindow import ModalWindow
from spug.curses.Listbox import Listbox
from spug.curses.KeyEvent import KeyEvent
from spug.curses.SimpleTextViewer import SimpleTextViewer

try:
   from cStringIO import StringIO
except ImportError:
   from StringIO import StringIO

import main

class ContentViewer(ModalWindow):
   
   def __init__(self, x0, y0, width, height, text):
      ModalWindow.__init__(self, x0, y0, width, height)
      self.viewer = SimpleTextViewer(self, 0, 0, width, height, text)
      self.__menu =  [ (113, 'Quit', self.terminate) ]
   
   def processEvent(self, evt):
      if isinstance(evt, KeyEvent):
         if evt.code == self.KEY_LEFT:
            self.terminate()
            return 1
         elif evt.code == 24:             # Ctrl-X
            main.menu(self.__menu, self)
      
      return ModalWindow.processEvent(self, evt)

class AttachmentListItem:
   
   def __init__(self, name, text, attachment):
      self.name = name
      self.text = text
      self.attachment = attachment
   
   def __str__(self):
      return self.text

class AttachmentsListbox(Listbox):
   
   def _itemAsString(self, item):
      return str(item)

class AttachmentsViewer(ModalWindow):

   def __addAttachments(self, msg, indent = 0):
      for att in msg.attachments:
         
         # get the content type information
         contentType, ctParms = att.getContentType()
         
         # initialize "name" to empty.
         name = ''
         
         # see if this is an attachment, if so, and it has a "name" parameter,
         # use that as the name
         if att.headers.has_key('content-disposition'):
            try:
               type, parms = att.headers.parseParms('content-disposition')
               if type == 'attachment':
                  if parms.has_key('filename'):
                     name = parms['filename']
                  elif parms.has_key('name'):
                     name = parms['name']
            except TypeError:
               pass
         
         # otherwise see if there is a name in the content type parms
         if not name and ctParms.has_key('name'):
            name = ctParms['name']
         
         # format an entry
         text = '%s%-40s (%s)' % (' ' * (indent * 3), name, contentType)
         
         self.list.append(AttachmentListItem(name, text, att))
         
         # recurse if there are embedded attachments
         if att.attachments:
            self.__addAttachments(att, indent + 1)

   def __init__(self, x0, y0, width, height, msg):
      ModalWindow.__init__(self, x0, y0, width, height)
      self.list = AttachmentsListbox(self, 0, 0, width, height, [])
      self.msg = msg
      
      self.__menu = \
         [
         (118, 'View attachment', self.view),
         (119, 'Write attachment', self.write),
         (113, 'Quit', self.quit),
         ]
      
      self.__addAttachments(self.msg)

   def view(self):
      item = self.list.selection()
      if not item:
         mainWin.showStatus('You must first select an attachment')
         return      

      # write the contents of the attachment to a string
      out = StringIO()
      item.attachment.writeTo(out)
      
      # display it
      x, y = self.getPos()
      ContentViewer(x, y, self._width, self._height, out.getvalue()).show(self)

   def write(self):
      item = self.list.selection()
      if not item:
         mainWin.showStatus('You must first select an attachment')
         return
      
      # get the filename, if it was empty, take that as a cancel
      fileName = main.mainWin.queryUser('Filename:', initValue = item.name)
      if not fileName:
         return
      
      try:
         encoding = \
            string.lower(item.attachment.headers['content-transfer-encoding'])
      except KeyError:
         encoding = None
      
      if encoding is None:
         open(fileName, 'wb').write(item.attachment.body)
         main.mainWin.showStatus('Attachment written as "%s"' %
                                 fileName
                                 )
      elif encoding == 'base64':
         import base64
         content = base64.decodestring(item.attachment.body)
         open(fileName, 'wb').write(content)
         main.mainWin.showStatus('Base64 Attachment written as "%s"' %
                                 fileName
                                 )
      elif encoding == '7bit':
         open(fileName, 'wb').write(item.attachment.body)
         main.mainWin.showStatus('7bit attachment written as "%s"' %
                                 fileName
                                 )
      else:
         main.mainWin.showStatus('Unknown attachment type: %s, writing as '
                                  'text' % encoding
                                 )
         item.attachment.writeTo(open(fileName, 'wb'))

   def quit(self):
      del self.__menu
      self.terminate()

   def processEvent(self, evt):
      if isinstance(evt, KeyEvent):
         if evt.code == 24:           # Ctrl-X
            main.menu(self.__menu, self)
            return 1
         elif evt.code == 10:         # Enter key
            self.view()
            return 1
         elif evt.code == self.KEY_LEFT:
            self.quit()
            return 1
         
      return ModalWindow.processEvent(self, evt)
